import argparse
import os.path
from StringIO import StringIO

from texttable import Texttable

from org import Org, Agenda

def argparser(subparsers):
    parser_agenda = subparsers.add_parser('agenda')
    parser_agenda.add_argument('-f', '--file',
                               dest='filename',
                               type=argparse.FileType('r'),
                               help='Org file to process')
    parser_agenda.add_argument('-n', '--ndays',
                               dest='ndays',
                               default=7,
                               type=int,
                               help='Number of days in the agenda')
    parser_agenda.set_defaults(func=agenda)

class Table1:
    def __init__(self):
        self._table = Texttable()
        
        width = int(os.getenv("COLUMNS", 180)) - 60
        self._table.set_cols_align(["r", "r", "l"])
        self._table.set_cols_width([25, 25, width])
        self._table.set_deco(0)
    
    def add_row(self, row):
        row = map(lambda x: x.encode('utf-8'), row)
        self._table.add_row(row)
    
    def draw(self):
        return self._table.draw()

class Table2:
    sep = '|'
    def __init__(self):
        self._buffer = StringIO()
    
    def add_row(self, row):
        print >>self._buffer, self.sep,
        for c in row:
            print >>self._buffer, c, self.sep,
        print >>self._buffer
    
    def draw(self):
        return self._buffer.getvalue().encode('utf-8')

def agenda(conf, args):
    sep = "|"
    print "Agenda:", args.ndays, "days"
    
    states = conf['states']
    
    table = Table1()
    
    if args.filename is None:
        if 'agenda' in conf and 'files' in conf['agenda']:
            filenames = map(os.path.expanduser, conf['agenda']['files'])
        else:
            filenames = []
    else:
        filenames = [args.filename]
    
    orgs = map(lambda x: Org(x, states), filenames)
    agenda = Agenda(orgs, args.ndays)
    
    for date in agenda:
        d = date.strftime("%A %e %B %Y")
        table.add_row([d, "", ""])
        
        for node in agenda[date]['active']:
            table.add_row(["", "", node.heading(stars = False)])
        
        for node in agenda[date]['scheduled']:
            delta = date - node.scheduled()
            if delta.days == 0:
                table.add_row(["", "", node.heading(stars = False)])
            else:
                table.add_row(["", "Sched.%ix" % delta.days, node.heading(stars = False)])
        
        for node in agenda[date]['deadline']:
            delta = node.deadline() - date
            if delta.days == 0:
                table.add_row(["", "DEADLINE", node.heading(stars = False)])
            else:
                table.add_row(["", "In %i d." % delta.days, node.heading(stars = False)])
    
    print table.draw()

